import calendar as _calendar
from datetime import date, timedelta, datetime

from django.contrib.auth.decorators import login_required, permission_required
from django.db.models import Count, F
from django.db.models import Q, Min, Max, Avg, StdDev
from django.db.models.functions import TruncMonth, ExtractDay
from django.shortcuts import render
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from . import Median, week_view
from ..models import RecurrentImport, Event


def home(request, cat=None):
    return week_view(request, home=True, cat=cat)


def thank_you(request):
    return render(request, "agenda_culturel/thank_you.html")


def mentions_legales(request):
    context = {
        "title": "Mentions légales",
        "static_content": "mentions_legales",
        "url_path": reverse_lazy("mentions_legales"),
    }
    return render(request, "agenda_culturel/page-single.html", context)


def about(request):
    rimports = (
        RecurrentImport.objects.filter(~Q(recurrence=RecurrentImport.RECURRENCE.NEVER))
        .order_by("name__unaccent")
        .all()
    )
    context = {
        "title": "À propos",
        "static_content": "about",
        "url_path": reverse_lazy("about"),
        "rimports": rimports,
    }
    return render(request, "agenda_culturel/page-rimports-list.html", context)


def moderation_rules(request):
    context = {
        "title": _("Moderation rules"),
        "static_content": "moderation_rules",
        "url_path": reverse_lazy("moderation_rules"),
    }
    return render(request, "agenda_culturel/page-single.html", context)


def import_requirements(request):
    context = {
        "title": _("Import requirements"),
        "static_content": "import_requirements",
        "url_path": reverse_lazy("import_requirements"),
    }
    return render(request, "agenda_culturel/page-single.html", context)


def statistics(request, pk=None):
    if pk is not None:
        rimport = RecurrentImport.objects.filter(pk=pk)
        source = rimport.values("source").first()["source"]
        qs = Event.objects.filter(import_sources__contains=[source])
    else:
        rimport = None
        qs = Event.objects

    stats = {}
    stats_months = {}
    first = {}
    last = {}

    ev_published = qs.filter(
        Q(status=Event.STATUS.PUBLISHED)
        & (
            Q(other_versions__isnull=True)
            | Q(other_versions__representative=F("pk"))
            | Q(other_versions__representative__isnull=True)
        )
    )

    for v in ["start_day", "created_date__date"]:
        after = 24
        last[v] = (
            date.today()
            if v == "created_date__date"
            else date.today() + timedelta(weeks=after)
        )
        last[v] = last[v].replace(
            day=_calendar.monthrange(last[v].year, last[v].month)[1]
        )

        r = 8 * 30
        if v == "start_day":
            r += after * 7
        first[v] = (last[v] - timedelta(days=r)).replace(day=1)

        ev_days = ev_published.annotate(day=F(v)).filter(
            Q(day__lte=last[v]) & Q(day__gte=first[v])
        )

        stats[v] = ev_days.values("day").annotate(total=Count("day")).order_by("day")

        stats_months[v] = (
            ev_days.annotate(month=TruncMonth("day"))
            .values("month")
            .annotate(total=Count("month"))
            .order_by("month")
        )

    nb_by_city = (
        ev_published.annotate(city=F("exact_location__city"))
        .filter(city__isnull=False)
        .values("city")
        .annotate(total=Count("city"))
        .order_by("-total")
    )

    limit = datetime.now() + timedelta(days=-30)

    stat_qs = qs.filter(start_day__gte=F("created_date")).annotate(
        foresight=ExtractDay(F("start_day") - F("created_date"))
    )

    statsa = stat_qs.filter().aggregate(
        minimum=Min("foresight"),
        maximum=Max("foresight"),
        mean=Avg("foresight"),
        median=Median("foresight"),
        stdev=StdDev("foresight"),
    )

    statsm = stat_qs.filter(created_date__gte=limit).aggregate(
        minimum=Min("foresight"),
        maximum=Max("foresight"),
        mean=Avg("foresight"),
        median=Median("foresight"),
        stdev=StdDev("foresight"),
    )

    stats_foresight = [
        [
            _(x),
            round(statsa[x], 2) if statsa[x] is not None else "-",
            round(statsm[x], 2) if statsm[x] is not None else "-",
        ]
        for x in statsa
    ]

    context = {
        "stats_by_startday": stats["start_day"],
        "stats_by_creation": stats["created_date__date"],
        "stats_months_by_startday": stats_months["start_day"],
        "stats_months_by_creation": stats_months["created_date__date"],
        "first_by_startday": first["start_day"],
        "last_by_startday": last["start_day"],
        "first_by_creation": first["created_date__date"],
        "last_by_creation": last["created_date__date"],
        "nb_by_city": nb_by_city,
        "stats_foresight": stats_foresight,
        "object": rimport.first() if rimport else None,
    }

    if pk is None:
        return render(request, "agenda_culturel/statistics.html", context)
    else:
        return render(request, "agenda_culturel/rimport-statistics.html", context)


@login_required(login_url="/accounts/login/")
@permission_required("agenda_culturel.view_event")
def activite(request):
    now = date.today()

    days = [now]
    while len(days) < 7 or days[-1].weekday() != 0:
        days.append(days[-1] + timedelta(days=-1))

    weeks = [days[-1]]
    for w in range(0, 8):
        weeks.append(weeks[-1] + timedelta(days=-7))

    daily_modifications = Event.get_count_modifications([(d, 1) for d in days])
    weekly_modifications = Event.get_count_modifications([(w, 7) for w in weeks])

    return render(
        request,
        "agenda_culturel/page-activity.html",
        {
            "daily_modifications": daily_modifications,
            "weekly_modifications": weekly_modifications,
        },
    )
